//---------------------------------------------------------------------
//  Algorithmic Conjurings @ http://www.coyotegulch.com
//
//  Itzam - An Embedded Database Engine
//
//  itzam_btree_debug.c
//
//---------------------------------------------------------------------
//
//  Copyright 2004, 2005 Scott Robert Ladd
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//      Free Software Foundation, Inc.
//      59 Temple Place - Suite 330
//      Boston, MA 02111-1307, USA.
//
//-----------------------------------------------------------------------
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//
//-----------------------------------------------------------------------

// library header file
#include "itzam_debug.h"

// Standard C library
#include <stdlib.h>

static void hex_dump(uint8_t * bytes, size_t length, FILE * output)
{
    size_t n, address = 0;
    
    for (n = 0; n < length; ++n)
    {
        if (0 == n % 16)
        {
            fprintf(output,"\n%8lx:",address);
            address += 16;
        }
        
        fprintf(output," %02x",(int)bytes[n]);
    }
    
    fprintf(output,"\n");
}

static void print_header(const itzam_datafile * datafile, FILE * output)
{
    fprintf(output,"datafile header information\n------------------------------------------------------------\n");
    fprintf(output,"FILE *                   = %10p\n",(void *)datafile->m_file);
    fprintf(output,"error_handler *          = %10p\n",(void *)datafile->m_error_handler);
    fprintf(output,"m_header.m_signature     = 0x%08lx\n",datafile->m_header.m_signature);
    fprintf(output,"m_header.m_version       = 0x%08lx\n",datafile->m_header.m_version);
    fprintf(output,"m_header.m_dellist_ref   = 0x%08lx\n",datafile->m_header.m_dellist_ref);
    fprintf(output,"------------------------------------------------------------\n");
}

static void print_records(const itzam_datafile * datafile, FILE * output)
{
    uint8_t * data;
    itzam_int count = 0;
    itzam_record_header header;
    itzam_ref ref = 0;
    
    fprintf(output,"\nrecords:\n");
    fprintf(output,"----------------------------------------------------------------------\n");
    
    fseek(datafile->m_file,(long)sizeof(itzam_datafile_header),SEEK_SET);
    
    while (1)
    {
        ref = ftell(datafile->m_file);
        
        // read record header
        if (0 == fread(&header,sizeof(header),1,datafile->m_file))
            break;
        
        ++count;
        
        // print header
        fprintf(output,"\n*** record #%ld ***\n",count);\
        fprintf(output,"    reference = 0x%08lx\n", ref);
        fprintf(output,"       status =   %8s\n", (header.m_flags & ITZAM_RECORD_IN_USE) ? "in use" : "deleted");
        fprintf(output,"         type =   %8s\n", (header.m_flags & ITZAM_RECORD_DELLIST) ? "dellist" : "normal");
        fprintf(output,"       length =   %8ld\n", header.m_length);
        fprintf(output,"record length =   %8ld\n", header.m_rec_size);
        
        // read data
        data = (uint8_t *)malloc(header.m_length);
        
        if (0 == fread(data,header.m_length,1,datafile->m_file))
            break;
        
        // print data
        hex_dump(data,header.m_length,output);
        free(data);
    }
    
    fprintf(output,"----------------------------------------------------------------------\n");
}

void analyze_datafile(itzam_datafile * datafile, FILE * output)
{
    fprintf(output,"\n-> BEGIN DATAFILE ANALYSIS\n\n");
    print_header(datafile,output);
    print_records(datafile,output);    
    fprintf(output,"\n<- END DATAFILE ANALYSIS\n\n");
}
