//---------------------------------------------------------------------
//  Algorithmic Conjurings @ http://www.coyotegulch.com
//
//  Itzam - An Embedded Database Engine
//
//  itzam_general.c
//
//---------------------------------------------------------------------
//
//  Copyright 2004, 2005 Scott Robert Ladd
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//      Free Software Foundation, Inc.
//      59 Temple Place - Suite 330
//      Boston, MA 02111-1307, USA.
//
//-----------------------------------------------------------------------
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//
//-----------------------------------------------------------------------

// library header file
#include "itzam.h"

// Standard C library
#include <stdlib.h>

// TODO: needs to be internationalized someday
static const char * ERROR_STRINGS [] =
{
    "invalid datafile signature",
    "invalid version",
    "can not open 64-bit datafile on 32-bit operating system",
    "write failed",
    "open failed",
    "read failed",
    "close failed",
    "seek failed",
    "tell failed",
    "duplicate remove",
    "flush failed",
    "rewrite record too small",
    "NULL allocation",
    "page not found",
    "lost key",
    "key not written",
    "key seek failed",
    "unable to remove key record",
    "record seek failed",
    "unable to remove data record",
    "list of deleted records could not be read",
    "list of deleted records could not be written",
    "iterator record count differs from database internal count",
    "rewrite over deleted record"
};

static const char * STATE_MESSAGES [] =
{
    "okay",
    "iterator at end",
    "iterator at beginning",
    "unknown state, possibly broken",
    "key not found",
    "duplicate key",
    "exceeded maximum file size on 32-bit system",
    "unable to write data record for index",
    "sizeof(size_t) smaller than required for file references; possibly 64-bit DB on 32-bit platform"
};

// pointer to default error handler
const char * itzam_get_state_description(itzam_state state)
{
    return STATE_MESSAGES[state];
}

static void default_error_handler(const char * function_name, itzam_error error)
{
    // display an error to stderr
    fprintf(stderr,"itzam error in %s: %s\n", function_name, ERROR_STRINGS[error]);

    // exit program
    exit(1);
}

static itzam_error_handler * itzam_default_error_handler = default_error_handler;

void itzam_set_default_error_handler(itzam_error_handler * error_handler)
{
    if (error_handler == NULL)
        itzam_default_error_handler = default_error_handler;
    else
        itzam_default_error_handler = error_handler;
}

itzam_error_handler * itzam_get_default_error_handler()
{
    return itzam_default_error_handler;
}
