//---------------------------------------------------------------------
//  Algorithmic Conjurings @ http://www.coyotegulch.com
//
//  Itzam - An Embedded Database Engine
//
//  itzam_data_testls.c
//
//---------------------------------------------------------------------
//
//  Copyright 2004, 2005 Scott Robert Ladd
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//      Free Software Foundation, Inc.
//      59 Temple Place - Suite 330
//      Boston, MA 02111-1307, USA.
//
//-----------------------------------------------------------------------
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//
//-----------------------------------------------------------------------

//----------------------------------------------------------
// Standard C
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#if !defined(_MSC_VER)
#include <fcntl.h>
#include <unistd.h>
#endif

//----------------------------------------------------------
// itzam
#include "../libitzam/itzam.h"
#include "../libitzam/itzam_debug.h"

static const char * FILENAME1 = "test1.dat";

//----------------------------------------------------------
//  Reports an itzam error
static void not_okay(itzam_state state)
{
    printf("\nProblematic Itzam state: %s\n",itzam_get_state_description(state));
    exit(EXIT_FAILURE);
}

//----------------------------------------------------------
//  Display program usage
static const char * USAGE =
    "\nusage: itzam_data_test filename\n\n"
    "    -help      this usage information\n"
    "\n";

static void show_usage()
{
    puts(USAGE);
}

//----------------------------------------------------------
// tests
void test_create_empty()
{
    itzam_datafile datafile;
    itzam_state    state;
    
    printf("test: creating empty datafile\n");
    
    // open datafile read-only
    state = itzam_datafile_create(&datafile,FILENAME1);

    if (state != ITZAM_OKAY)
        not_okay(state);

    // close database
    itzam_datafile_close(&datafile);
    printf("      done\n");
}

void test_add()
{
    itzam_datafile datafile;
    itzam_state    state;
    
    printf("test: adding one record\n");
    
    // open datafile read-only
    state = itzam_datafile_open(&datafile,FILENAME1,false);

    if (state != ITZAM_OKAY)
        not_okay(state);
    
    // add a record
    uint32_t record = 1;
    
    itzam_datafile_write(&datafile,&record,sizeof(record),ITZAM_NULL_REF);

    // close database
    itzam_datafile_close(&datafile);
    printf("      done\n");
}

void test_remove()
{
    itzam_datafile datafile;
    itzam_state    state;
    
    printf("test: removing first record\n");
    
    // open datafile read-only
    state = itzam_datafile_open(&datafile,FILENAME1,false);

    if (state != ITZAM_OKAY)
        not_okay(state);
    
    // rewind to first record
    state = itzam_datafile_rewind(&datafile);
    
    if (state != ITZAM_OKAY)
        not_okay(state);
    
    // add a record
    itzam_datafile_remove(&datafile);

    // close database
    itzam_datafile_close(&datafile);
    printf("      done\n");
}

//----------------------------------------------------------
// main program
int main(int argc, char * argv[])
{
    // test code
    size_t         n;
    int            i;

    // parse arguments
    for (i = 1; i < argc; ++i)
    {
        if (0 == strcmp(argv[i],"-help"))
        {
            show_usage();
            exit(EXIT_FAILURE);
        }
    }
    
    // run tests
    test_create_empty();
    test_add();
    test_remove();
    test_add();
    test_add();
    test_remove();
    test_add();

    // done
    return EXIT_SUCCESS;
}
